

set -ex



$PREFIX/bin/llvm-config --libs
$PREFIX/bin/llc -version
test -f $PREFIX/include/llvm/Pass.h
test -f $PREFIX/lib/libLLVMSupport.a
test -f $PREFIX/lib/libLLVMCore.a
$PREFIX/bin/opt -dot-cfg cfg_test.ll &>/dev/null
python test_cfg_dot.py
ld.lld --version
exit 0
